@@ -678,7 +678,7 @@ use Carp;
 
 use DBI qw(dbi_time dbi_profile dbi_profile_merge_nodes dbi_profile_merge);
 
-$VERSION = sprintf("2.%06d", q$Revision: 13581 $ =~ /(\d+)/o);
+$VERSION = sprintf("2.%06d", q$Revision: 13969 $ =~ /(\d+)/o);
 
 
 @ISA = qw(Exporter);
@@ -721,7 +721,7 @@ sub _auto_new {
     $arg =~ s/^DBI::/2\/DBI::/
         and carp "Automatically changed old-style DBI::Profile specification to $arg";
 
-    # it's a path/module/arg/arg/arg list
+    # it's a path/module/k1:v1:k2:v2:... list
     my ($path, $package, $args) = split /\//, $arg, 3;
     my @args = (defined $args) ? split(/:/, $args, -1) : ();
     my @Path;
@@ -926,11 +926,14 @@ sub on_destroy {
     return unless $self->{Data};
     my $detail = $self->format();
     $ON_DESTROY_DUMP->($detail) if $detail;
+    $self->{Data} = undef;
 }
 
 sub DESTROY {
     my $self = shift;
     local $@;
+    DBI->trace_msg("profile data DESTROY\n",0)
+        if (($self->{Trace}||0) >= 2);
     eval { $self->on_destroy };
     if ($@) {
         chomp $@;
